/**
 * Theme Name: errigal
 * Theme URI: https://projecterrigal.com/themes/errigal/
 * Author: Project errigal
 * Author URI: https://projecterrigal.com/
 * Version: 1.0.4
 */
( function() {
	'use strict';

	const ATTRIBUTE_NAME_ARIA_INVALID = 'aria-invalid';
	const ATTRIBUTE_NAME_ARIA_LABEL = 'aria-label';
	const ATTRIBUTE_NAME_ARIA_HIDDEN = 'aria-hidden';
	const ATTRIBUTE_VALUE_TRUE = 'true';
	const CHARACTER_PERIOD = '.';
	const CLASS_COMPONENT_INPUT_FIELD = 'input-field';
	const CLASS_SHARED_ACTIVE = 'active';
	const CLASS_SHARED_ERROR = 'error';
	const CLASS_SHARED_ERROR_MESSAGE = 'error-message';
	const EMPTY_STRING = '';
	const EVENT_TYPE_CLICK = 'click';
	const EVENT_TYPE_SUBMIT = 'submit';
	const ID_COMMENT_FORM = 'commentform';
	const ID_COMMENT_FORM_AUTHOR = 'author';
	const ID_COMMENT_FORM_EMAIL = 'email';
	const ID_COMMENT_FORM_COMMENT = 'comment';
	const ID_COMMENT_FORM_SUBMIT = 'submit';
	const NODE_TYPE_HTML_ELEMENT = 1;
	const NODE_TYPE_DOCUMENT = 9;
	const TAG_DIV = 'div';
	const TAG_INPUT = 'input';
	const TAG_LABEL = 'label';
	const TAG_TEXTAREA = 'textarea';

	function getCommentFormAuthorField() {
		return document.getElementById( ID_COMMENT_FORM_AUTHOR );
	}

	function getCommentFormCommentField() {
		return document.getElementById( ID_COMMENT_FORM_COMMENT );
	}

	function getCommentFormEmailField() {
		return document.getElementById( ID_COMMENT_FORM_EMAIL );
	}

	function getFirstErrorCommentFormField() {
		return document.querySelector( `#${ ID_COMMENT_FORM } .${ CLASS_SHARED_ERROR }` );
	}

	function getInputFields() {
		return document.querySelectorAll( `.${ CLASS_COMPONENT_INPUT_FIELD } > ${ TAG_INPUT }, .${ CLASS_COMPONENT_INPUT_FIELD } > ${ TAG_TEXTAREA }` );
	}

	function addClass( element, className ) {
		if ( isHTMLElement( element ) && element.classList && isString( className ) && ! isNullOrEmpty( className ) ) {
			element.classList.add( className );
		}
	}

	function matches( element, selectorString, isTag ) {
		if ( ! isHTMLElement( element ) || ! isString( selectorString ) || isNullOrEmpty( selectorString ) ) {
			return false;
		}

		if ( ! sanitiseBoolean( isTag ) && selectorString.indexOf( CHARACTER_PERIOD ) !== 0 ) {
			selectorString = CHARACTER_PERIOD + selectorString;
		}

		return element.matches ? element.matches( selectorString ) : element.msMatchesSelector( selectorString );
	}

	function removeClass( element, className ) {
		if ( isHTMLElement( element ) && element.classList && isString( className ) && ! isNullOrEmpty( className ) ) {
			element.classList.remove( className );
		}
	}

	function getElementByTagName( qualifiedName, rootElement ) {
		if ( ! isString( qualifiedName ) || isNullOrEmpty( qualifiedName ) ) {
			return null;
		}

		if ( ! rootElement ) {
			rootElement = document;
		}

		return isHTMLElement( rootElement ) ? rootElement.getElementsByTagName( qualifiedName )[ 0 ] : null;
	}

	function sanitiseBoolean( value ) {
		return isBoolean( value ) ? value : false;
	}

	function sanitiseString( value ) {
		if ( ! isString( value ) ) {
			return EMPTY_STRING;
		}

		value = value.replace( /<script[^>]*?>.*?<\/script>/gi, EMPTY_STRING );
		value = value.replace( /<\/?\w(?:[^"'>]|"[^"]*"|'[^']*')*>/g, EMPTY_STRING );
		value = value.replace( /&/g, '&amp;' );
		value = value.replace( /</g, '&lt;' );
		value = value.replace( />/g, '&gt;' );
		value = value.replace( /"/g, '&quot;' );
		value = value.replace( /'/g, '&#39;' );

		return value;
	}

	function isBoolean( value ) {
		return value === false || value === true;
	}

	function isHTMLElement( element, className ) {
		if ( ! element ) {
			return false;
		}

		const IS_HTML_ELEMENT = element.nodeType === NODE_TYPE_HTML_ELEMENT && element instanceof HTMLElement;
		const IS_DOCUMENT = element.nodeType === NODE_TYPE_DOCUMENT && element instanceof Document;
		const IS_HTML_ELEMENT_OR_DOCUMENT = IS_HTML_ELEMENT || IS_DOCUMENT;

		if ( isString( className ) && ! isNullOrEmpty( className ) ) {
			return IS_HTML_ELEMENT_OR_DOCUMENT && matches( element, className );
		}

		return IS_HTML_ELEMENT_OR_DOCUMENT;
	}

	function isNullOrEmpty( value ) {
		return value === null || value === undefined || value === EMPTY_STRING;
	}

	function isString( value ) {
		return typeof value === 'string';
	}

	function initialiseInputFieldLabelPositions() {
		const INPUT_FIELDS = getInputFields();

		for ( let i = 0; i < INPUT_FIELDS.length; i++ ) {
			if ( ! isNullOrEmpty( INPUT_FIELDS[ i ].value ) ) {
				moveInputFieldNameToLegend( INPUT_FIELDS[ i ] );
			}
		}
	}

	function getLabel( element ) {
		if ( ! isHTMLElement( element ) || ! isInputField( element ) ) {
			return null;
		}

		const INPUT_FIELD_CONTAINER = element.parentElement;

		return INPUT_FIELD_CONTAINER ? getElementByTagName( TAG_LABEL, INPUT_FIELD_CONTAINER ) : null;
	}

	function isInputField( element ) {
		if ( ! isHTMLElement( element ) || ! ( matches( element, TAG_INPUT, true ) || matches( element, TAG_TEXTAREA, true ) ) ) {
			return false;
		}

		return isHTMLElement( element.parentElement, CLASS_COMPONENT_INPUT_FIELD );
	}

	function moveInputFieldNameToLegend( element ) {
		if ( ! isHTMLElement( element ) || ! isInputField( element ) ) {
			return;
		}

		const LABEL = getLabel( element );

		if ( LABEL ) {
			addClass( LABEL, CLASS_SHARED_ACTIVE );
		}
	}

	function addCommentValidationEvents() {
		document.addEventListener( EVENT_TYPE_CLICK, ( event ) => {
			if ( isSubmit( event.target ) && ! validateCommentForm() ) {
				event.preventDefault();
				focusFirstErrorCommentFormField();
			}
		} );
		document.addEventListener( EVENT_TYPE_SUBMIT, ( event ) => {
			if ( isCommentForm( event.target ) ) {
				initialiseInputFieldLabelPositions();
			}
		} );
	}

	function addInputFieldError( inputField, errorMessage ) {
		if ( ! isHTMLElement( inputField ) || ! hasErrorMessage( errorMessage ) ) {
			return;
		}

		const ERROR_MESSAGE_TEXT = sanitiseString( errorMessage );

		addClass( inputField, CLASS_SHARED_ERROR );
		inputField.setAttribute( ATTRIBUTE_NAME_ARIA_LABEL, ERROR_MESSAGE_TEXT );
		inputField.setAttribute( ATTRIBUTE_NAME_ARIA_INVALID, ATTRIBUTE_VALUE_TRUE );

		const ERROR_MESSAGE = document.createElement( TAG_DIV );

		ERROR_MESSAGE.textContent = ERROR_MESSAGE_TEXT;
		addClass( ERROR_MESSAGE, CLASS_SHARED_ERROR_MESSAGE );
		ERROR_MESSAGE.setAttribute( ATTRIBUTE_NAME_ARIA_HIDDEN, ATTRIBUTE_VALUE_TRUE );
		inputField.parentNode.appendChild( ERROR_MESSAGE );
	}

	function focusFirstErrorCommentFormField() {
		const FIRST_ERROR_COMMENT_FORM_FIELD = getFirstErrorCommentFormField();

		if ( FIRST_ERROR_COMMENT_FORM_FIELD ) {
			FIRST_ERROR_COMMENT_FORM_FIELD.focus();
		}
	}

	function hasErrorMessage( errorMessage ) {
		return isString( errorMessage ) && ! isNullOrEmpty( errorMessage );
	}

	function isCommentForm( element ) {
		return isHTMLElement( element ) && element.id === ID_COMMENT_FORM;
	}

	function isSubmit( element ) {
		return isHTMLElement( element ) && element.id === ID_COMMENT_FORM_SUBMIT;
	}

	function removeInputFieldError( inputField ) {
		if ( ! isHTMLElement( inputField ) || ! isHTMLElement( inputField.parentNode ) ) {
			return;
		}

		removeClass( inputField, CLASS_SHARED_ERROR );
		inputField.removeAttribute( ATTRIBUTE_NAME_ARIA_LABEL );
		inputField.removeAttribute( ATTRIBUTE_NAME_ARIA_INVALID );

		const ERROR_MESSAGE = inputField.parentNode.querySelector( `${ TAG_DIV }.${ CLASS_SHARED_ERROR_MESSAGE }` );

		if ( ERROR_MESSAGE ) {
			ERROR_MESSAGE.parentNode.removeChild( ERROR_MESSAGE );
		}
	}

	function validateCommentForm() {
		let hasComment, hasName, hasEmail, isEmailFormatValid;

		hasComment = hasName = hasEmail = isEmailFormatValid = true;

		const COMMENT_FIELD = getCommentFormCommentField();
		const AUTHOR_FIELD = getCommentFormAuthorField();
		const EMAIL_FIELD = getCommentFormEmailField();

		if ( COMMENT_FIELD ) {
			hasComment = validateFieldFilled(
				COMMENT_FIELD,
				// eslint-disable-next-line no-undef
				ERRIGAL_COMMENT_FORM_ERROR.CommentRequired
			);
		}

		if ( AUTHOR_FIELD && AUTHOR_FIELD.required ) {
			hasName = validateFieldFilled(
				AUTHOR_FIELD,
				// eslint-disable-next-line no-undef
				ERRIGAL_COMMENT_FORM_ERROR.AuthorRequired
			);
		}

		if ( EMAIL_FIELD && EMAIL_FIELD.required ) {
			hasEmail = validateFieldFilled(
				EMAIL_FIELD,
				// eslint-disable-next-line no-undef
				ERRIGAL_COMMENT_FORM_ERROR.EmailAddressRequired
			);

			if ( hasEmail ) {
				isEmailFormatValid = validateEmailAddressFormat(
					EMAIL_FIELD,
					// eslint-disable-next-line no-undef
					ERRIGAL_COMMENT_FORM_ERROR.EmailAddressInvalid
				);
			}
		}

		return hasComment && hasName && hasEmail && isEmailFormatValid;
	}

	function validateEmailAddressFormat( inputField, errorMessage ) {
		if ( ! isHTMLElement( inputField ) || ! hasErrorMessage( errorMessage ) ) {
			return true;
		}

		removeInputFieldError( inputField );

		const REGEX_EMAIL = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;

		if ( ! REGEX_EMAIL.test( inputField.value.toLowerCase() ) ) {
			addInputFieldError( inputField, errorMessage );

			return false;
		}

		return true;
	}

	function validateFieldFilled( inputField, errorMessage ) {
		if ( ! isHTMLElement( inputField ) || ! hasErrorMessage( errorMessage ) ) {
			return true;
		}

		removeInputFieldError( inputField );

		if ( inputField.value.trim().length === 0 ) {
			addInputFieldError( inputField, errorMessage );

			return false;
		}

		return true;
	}

	addCommentValidationEvents();
}() );
