/**
 * Theme Name: errigal
 * Theme URI: https://projecterrigal.com/themes/errigal/
 * Author: Project errigal
 * Author URI: https://projecterrigal.com/
 * Version: 1.0.4
 */
( function() {
	'use strict';

	const CALLBACK_INITIAL_ATTEMPT_COUNT = 0;
	const CALLBACK_INTERVAL = 50;
	const CALLBACK_MAX_ATTEMPT_COUNT = 20;
	const CHARACTER_PERIOD = '.';
	const CLASS_WP_BLOCK_EMBED_YOUTUBE = 'wp-block-embed-youtube';
	const EMPTY_STRING = '';
	const EVENT_TYPE_LOAD = 'load';
	const EVENT_TYPE_RESIZE = 'resize';
	const NODE_TYPE_HTML_ELEMENT = 1;
	const NODE_TYPE_DOCUMENT = 9;
	const TAG_IFRAME = 'iframe';
	const UNIT_PIXEL = 'px';

	function getIframes() {
		return document.getElementsByTagName( TAG_IFRAME );
	}

	function matches( element, selectorString, isTag ) {
		if ( ! isHTMLElement( element ) || ! isString( selectorString ) || isNullOrEmpty( selectorString ) ) {
			return false;
		}

		if ( ! sanitiseBoolean( isTag ) && selectorString.indexOf( CHARACTER_PERIOD ) !== 0 ) {
			selectorString = CHARACTER_PERIOD + selectorString;
		}

		return element.matches ? element.matches( selectorString ) : element.msMatchesSelector( selectorString );
	}

	function sanitiseBoolean( value ) {
		return isBoolean( value ) ? value : false;
	}

	function isBoolean( value ) {
		return value === false || value === true;
	}

	function isHTMLCollection( element ) {
		return element instanceof HTMLCollection;
	}

	function isHTMLElement( element, className ) {
		if ( ! element ) {
			return false;
		}

		const IS_HTML_ELEMENT = element.nodeType === NODE_TYPE_HTML_ELEMENT && element instanceof HTMLElement;
		const IS_DOCUMENT = element.nodeType === NODE_TYPE_DOCUMENT && element instanceof Document;
		const IS_HTML_ELEMENT_OR_DOCUMENT = IS_HTML_ELEMENT || IS_DOCUMENT;

		if ( isString( className ) && ! isNullOrEmpty( className ) ) {
			return IS_HTML_ELEMENT_OR_DOCUMENT && matches( element, className );
		}

		return IS_HTML_ELEMENT_OR_DOCUMENT;
	}

	function isNullOrEmpty( value ) {
		return value === null || value === undefined || value === EMPTY_STRING;
	}

	function isNumber( value ) {
		return typeof value === 'number' && isFinite( value );
	}

	function isString( value ) {
		return typeof value === 'string';
	}

	function addEmbeddedIframeEventsInEditor() {
		window.addEventListener( EVENT_TYPE_LOAD, () => {
			setIframeSizesInEditor();
		} );
		window.addEventListener( EVENT_TYPE_RESIZE, () => {
			setIframeSizesInEditor();
		} );
	}

	function calculateAspectRatio( width, height ) {
		if ( isNaN( width ) || isNaN( height ) ) {
			return null;
		}

		return parseFloat( width ) / parseFloat( height );
	}

	function getIframeHeight( iframe ) {
		if ( ! shouldResizeIframe( iframe ) || isNaN( iframe.width ) || isNaN( iframe.height ) ) {
			return;
		}

		const PARENT_WIDTH = getParentWidth( iframe );
		const ASPECT_RATIO = calculateAspectRatio( iframe.width, iframe.height );

		if ( ! isNumber( PARENT_WIDTH ) || ! isNumber( ASPECT_RATIO ) ) {
			return 0;
		}

		return Math.round( PARENT_WIDTH / ASPECT_RATIO );
	}

	function getParentWidth( element ) {
		if ( ! isHTMLElement( element ) || ! isHTMLElement( element.parentElement ) ) {
			return null;
		}

		const WIDTH = window.getComputedStyle( element.parentElement, null ).width;

		if ( WIDTH === null || isNaN( parseFloat( WIDTH ) ) ) {
			return null;
		}

		return parseFloat( WIDTH );
	}

	function isIframe( element ) {
		return isHTMLElement( element ) && element.tagName.toLowerCase() === TAG_IFRAME;
	}

	function isLegacyYouTubeIframe( element ) {
		if ( ! isIframe( element ) ) {
			return false;
		}

		const WRAPPER = element.parentElement;
		const FIGURE = WRAPPER ? WRAPPER.parentElement : null;

		if ( ! FIGURE ) {
			return false;
		}

		return isHTMLElement( FIGURE, CLASS_WP_BLOCK_EMBED_YOUTUBE );
	}

	function processIframes( iframes ) {
		if ( ! isHTMLCollection( iframes ) ) {
			return;
		}

		for ( let i = 0; i < iframes.length; i++ ) {
			const IFRAME = iframes[ i ];

			setIframeSize( IFRAME );
		}
	}

	function setIframeSize( iframe ) {
		if ( ! shouldResizeIframe( iframe ) || isNaN( iframe.width ) || isNaN( iframe.height ) ) {
			return;
		}

		const IFRAME_HEIGHT = getIframeHeight( iframe );

		if ( isNumber( IFRAME_HEIGHT ) && IFRAME_HEIGHT > 0 ) {
			iframe.style.width = '100%';
			iframe.style.height = `${ IFRAME_HEIGHT }${ UNIT_PIXEL }`;
		}
	}

	function setIframeSizesInEditor() {
		// eslint-disable-next-line prefer-const
		let attemptCount = CALLBACK_INITIAL_ATTEMPT_COUNT;
		const IFRAMES = getIframes();

		setIframeSizesInEditorCallback( IFRAMES, attemptCount );
	}

	function setIframeSizesInEditorCallback( iframes, attemptCount ) {
		if ( isHTMLCollection( iframes ) && iframes.length > 0 ) {
			processIframes( iframes );

			return;
		}

		if ( attemptCount === CALLBACK_MAX_ATTEMPT_COUNT ) {
			return;
		}

		attemptCount++;

		setTimeout( () => {
			const IFRAMES = getIframes();

			setIframeSizesInEditorCallback( IFRAMES, attemptCount );
		}, CALLBACK_INTERVAL );
	}

	function shouldResizeIframe( iframe ) {
		return isIframe( iframe ) && ! isLegacyYouTubeIframe( iframe );
	}

	addEmbeddedIframeEventsInEditor();
}() );
