<?php
/**
 * Admin Class
 *
 * @package errigal
 * @since 1.0.4
 */

/**
 * This class is in charge of handling admin functionalities.
 *
 * @since 1.0.4
 */
class Errigal_Admin {
	/**
	 * Nonce: Admin.
	 *
	 * @since 1.0.4
	 *
	 * @var string
	 */
	private const NONCE_ADMIN = 'errigal_admin_nonce';

	/**
	 * User meta key: Welcome admin notice dismissed.
	 *
	 * @since 1.0.4
	 *
	 * @var string
	 */
	private $key_notice_dismissed_welcome;

	/**
	 * Constructor. Instantiates the object.
	 *
	 * @since 1.0.4
	 */
	public function __construct() {
		$this->key_notice_dismissed_welcome = sanitize_key( sanitize_key( wp_get_theme()->get( 'Name' ) ) . '_admin_notice_dismissed_welcome' );

		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_scripts' ) );

		add_action( 'admin_notices', array( $this, 'show_notice_welcome' ) );

		add_action( 'wp_ajax_errigal_dismiss_admin_notice_welcome', array( $this, 'dismiss_admin_notice_welcome' ) );
	}

	/**
	 * Dismisses welcome admin notice.
	 *
	 * @since 1.0.4
	 *
	 * @return void
	 */
	public function dismiss_admin_notice_welcome() {
		if ( ! isset( $_POST['nonce'] ) ||
			! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), sanitize_key( self::NONCE_ADMIN ) )
		) {
			wp_die();
		}

		update_user_meta(
			absint( get_current_user_id() ),
			sanitize_key( $this->key_notice_dismissed_welcome ),
			sanitize_text_field( wp_get_theme()->get( 'Version' ) )
		);

		wp_die();
	}

	/**
	 * Enqueues admin scripts.
	 *
	 * @since 1.0.4
	 *
	 * @global bool $is_IE Whether browser is Internet Explorer.
	 *
	 * @return void
	 */
	public function enqueue_admin_scripts() {
		global $is_IE;

		$admin_script = 'errigal-admin';

		// Enqueues admin style.
		wp_enqueue_style(
			'errigal-admin-style',
			esc_url( get_stylesheet_directory_uri() . '/assets/css/admin-' . ( $is_IE ? 'ie' : 'style' ) . '.min.css' ),
			array(),
			esc_attr( errigal_get_stylesheet_version() )
		);

		// Enqueues admin script.
		wp_enqueue_script(
			esc_attr( $admin_script ),
			esc_url( errigal_get_script_uri( 'admin.min.js' ) ),
			array( 'wp-i18n', 'jquery' ),
			esc_attr( errigal_get_script_version() ),
			true
		);

		wp_localize_script(
			esc_attr( $admin_script ),
			'ERRIGAL_ADMIN',
			array(
				'admin_media_modal_title'              => esc_html__( 'Select or upload an image file', 'errigal' ),
				'admin_media_modal_button_text_select' => esc_html__( 'Select', 'errigal' ),
				'nonce'                                => wp_create_nonce( sanitize_key( self::NONCE_ADMIN ) ),
			)
		);
	}

	/**
	 * Shows admin notice: Welcome.
	 *
	 * @since 1.0.4
	 *
	 * @global string $pagenow The filename of the current screen.
	 *
	 * @return void
	 */
	public function show_notice_welcome() {
		global $pagenow;

		$dismissed_version = get_user_meta(
			absint( get_current_user_id() ),
			sanitize_key( $this->key_notice_dismissed_welcome ),
			true
		);
		$current_version   = wp_get_theme()->get( 'Version' );

		if ( $current_version === $dismissed_version ) {
			return;
		}

		if ( isset( $dismissed_version ) || $current_version !== $dismissed_version ) {
			delete_user_meta(
				absint( get_current_user_id() ),
				sanitize_key( $this->key_notice_dismissed_welcome )
			);
		}

		if ( 'index.php' === $pagenow || 'themes.php' === $pagenow ) {
			get_template_part( 'template-parts/admin/notice-welcome' );
		}
	}
}

new Errigal_Admin();
