<?php
/**
 * Custom Colour Scheme Class
 *
 * @package errigal
 * @since 1.0.0
 */

/**
 * This class is in charge of colour scheme customisation via customiser.
 *
 * @since 1.0.0
 *
 * @see Errigal_Custom_Base
 */
class Errigal_Custom_Colour_Scheme extends Errigal_Custom_Base {
	/**
	 * Setting ID: Colour scheme.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const ID_COLOUR_SCHEME = 'colour_scheme';

	/**
	 * Default setting value: Colour scheme.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const DEFAULT_COLOUR_SCHEME = '';

	/**
	 * Constructor. Instantiates the object.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		add_action( 'customize_register', array( $this, 'register' ) );

		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_style' ) );

		add_action( 'after_setup_theme', array( $this, 'enqueue_editor_style' ) );
	}

	/**
	 * Registers customiser options.
	 *
	 * @since 1.0.0
	 *
	 * @param WP_Customize_Manager $wp_customise Theme customiser object.
	 * @return void
	 */
	public function register( $wp_customise ) {
		$wp_customise->add_setting(
			self::ID_COLOUR_SCHEME,
			array(
				'default'           => self::DEFAULT_COLOUR_SCHEME,
				'transport'         => 'postMessage',
				'sanitize_callback' => self::SANITIZE_CALLBACK_SELECT,
			)
		);

		$wp_customise->add_control(
			self::ID_COLOUR_SCHEME,
			array(
				'priority' => self::CONTROL_PRIORITY_APPEARANCE_COLOUR_SCHEME,
				'section'  => self::SECTION_APPEARANCE,
				'label'    => esc_html__( 'Website Colour Scheme', 'errigal' ),
				'type'     => 'select',
				'choices'  => $this->get_colour_schemes(),
			)
		);
	}

	/**
	 * Enqueues style.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function enqueue_style() {
		wp_enqueue_style(
			esc_attr( errigal_get_stylesheet_name() ),
			esc_url( $this->get_stylesheet_uri() ),
			array(),
			esc_attr( errigal_get_stylesheet_version() )
		);
	}

	/**
	 * Enqueues editor style.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function enqueue_editor_style() {
		add_theme_support( 'editor-styles' );

		$is_editor_style = true;

		add_editor_style( $this->get_stylesheet_uri( $is_editor_style ) );
	}

	/**
	 * Gets colour schemes.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	protected function get_colour_schemes() {
		return array(
			self::DEFAULT_COLOUR_SCHEME => esc_html__( 'Sky Blue', 'errigal' ),
			'mono'                      => esc_html__( 'Mono', 'errigal' ),
		);
	}

	/**
	 * Gets stylesheet URI for current theme.
	 *
	 * @since 1.0.0
	 *
	 * @global bool $is_IE Whether browser is Internet Explorer.
	 *
	 * @param bool $is_editor_style Optional. Whether stylesheet is for editor. Default false.
	 * @return string
	 */
	private function get_stylesheet_uri( $is_editor_style = false ) {
		global $is_IE;

		$colour_scheme = get_theme_mod( self::ID_COLOUR_SCHEME, self::DEFAULT_COLOUR_SCHEME );

		$file_name = sprintf(
			esc_attr( '%s%s%s.min.css' ),
			esc_attr( $is_editor_style ? 'editor-' : '' ),
			esc_attr( $is_IE ? 'ie' : 'style' ),
			esc_attr( self::DEFAULT_COLOUR_SCHEME === $colour_scheme ? $colour_scheme : '-' . $colour_scheme )
		);

		return get_stylesheet_directory_uri() . '/assets/css/' . $file_name;
	}
}

new Errigal_Custom_Colour_Scheme();
