<?php
/**
 * Custom Footer Class
 *
 * @package errigal
 * @since 1.0.0
 */

/**
 * This class is in charge of site footer customisation via customiser.
 *
 * @since 1.0.0
 *
 * @see Errigal_Custom_Base
 */
class Errigal_Custom_Footer extends Errigal_Custom_Base {
	/**
	 * Setting ID: Copyright text.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const ID_COPYRIGHT_TEXT = 'footer_copyright_text';

	/**
	 * Shortcode: Copyright.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const SHORTCODE_COPYRIGHT = '[copyright]';

	/**
	 * Shortcode: Current year.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const SHORTCODE_CURRENT_YEAR = '[current_year]';
	/**
	 * Shortcode: Site title.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const SHORTCODE_SITE_TITLE = '[site_title]';
	/**
	 * Shortcode: Theme link.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const SHORTCODE_THEME_LINK = '[theme_link]';

	/**
	 * Copyright sign.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const COPYRIGHT_SIGN = '&copy;';

	/**
	 * Constructor. Instantiates the object.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		add_action( 'customize_register', array( $this, 'register' ) );
	}

	/**
	 * Gets copyright text.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	public static function get_copyright_text() {
		$copyright_text = get_theme_mod( self::ID_COPYRIGHT_TEXT, esc_html( self::get_default_copyright_text() ) );

		$copyright_text = str_replace( self::SHORTCODE_COPYRIGHT, self::COPYRIGHT_SIGN, $copyright_text );
		$copyright_text = str_replace( self::SHORTCODE_CURRENT_YEAR, esc_html( self::get_current_year() ), $copyright_text );
		$copyright_text = str_replace( self::SHORTCODE_SITE_TITLE, esc_html( self::get_site_title() ), $copyright_text );
		$copyright_text = str_replace( self::SHORTCODE_THEME_LINK, wp_kses_post( self::get_theme_link() ), $copyright_text );

		return wp_kses_post( $copyright_text );
	}

	/**
	 * Gets default copyright text.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	private static function get_default_copyright_text() {
		return sprintf(
			'%1$s %2$s %3$s | %4$s',
			self::SHORTCODE_COPYRIGHT,
			self::SHORTCODE_CURRENT_YEAR,
			self::SHORTCODE_SITE_TITLE,
			self::SHORTCODE_THEME_LINK
		);
	}

	/**
	 * Gets current year.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	private static function get_current_year() {
		return date_i18n( 'Y' );
	}

	/**
	 * Gets site title.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	private static function get_site_title() {
		return get_bloginfo( 'name' );
	}

	/**
	 * Gets theme link.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	private static function get_theme_link() {
		$theme_uri  = wp_get_theme()->get( 'ThemeURI' );
		$theme_name = wp_get_theme()->get( 'Name' );

		if ( empty( $theme_uri ) || empty( $theme_name ) ) {
			return '';
		}

		return sprintf(
			/* translators: %s: Link to errigal theme web site. */
			esc_html__( 'Built with %s', 'errigal' ),
			'<a class="site-footer__theme-link" href="' . esc_url( $theme_uri ) . '" target="_blank" rel="nofollow noopener">' . esc_html( $theme_name ) . '</a>'
		);
	}

	/**
	 * Registers customiser options.
	 *
	 * @since 1.0.0
	 *
	 * @param WP_Customize_Manager $wp_customise Theme customiser object.
	 * @return void
	 */
	public function register( $wp_customise ) {
		$wp_customise->add_setting(
			self::ID_COPYRIGHT_TEXT,
			array(
				'default'           => esc_html( self::get_default_copyright_text() ),
				'transport'         => 'postMessage',
				'sanitize_callback' => self::SANITIZE_CALLBACK_TEXTAREA_HTML,
			)
		);

		$wp_customise->add_control(
			self::ID_COPYRIGHT_TEXT,
			array(
				'section'     => self::SECTION_FOOTER,
				'label'       => esc_html__( 'Display Copyright Text', 'errigal' ),
				'description' => sprintf(
					wp_kses_post( '%1$s<br><ul>%2$s%3$s%4$s%5$s</ul>%6$s' ),
					esc_html__( 'You can use the shortcodes below.', 'errigal' ),
					sprintf( '<li>%1$s = %2$s</li>', self::SHORTCODE_COPYRIGHT, self::COPYRIGHT_SIGN ),
					sprintf( '<li>%1$s = %2$s</li>', self::SHORTCODE_CURRENT_YEAR, esc_html( self::get_current_year() ) ),
					sprintf( '<li>%1$s = %2$s</li>', self::SHORTCODE_SITE_TITLE, esc_html( self::get_site_title() ) ),
					empty( self::get_theme_link() ) ? '' : sprintf( '<li>%1$s = %2$s</li>', self::SHORTCODE_THEME_LINK, wp_kses_post( self::get_theme_link() ) ),
					esc_html__( 'You can also use basic HTML tags.', 'errigal' )
				),
				'type'        => 'textarea',
			)
		);

		$wp_customise->selective_refresh->add_partial(
			self::ID_COPYRIGHT_TEXT,
			array(
				'selector'        => '.site-footer__copyright-text',
				'render_callback' => function() {
					echo wp_kses_post( self::get_copyright_text() );
				},
			)
		);
	}
}

new Errigal_Custom_Footer();
