<?php
/**
 * Custom Site Header Colour Class
 *
 * @package errigal
 * @since 1.0.0
 */

/**
 * This class is in charge of site header colour customisation via customiser.
 *
 * @since 1.0.0
 *
 * @see Errigal_Custom_Base
 */
class Errigal_Custom_Site_Header_Colour extends Errigal_Custom_Base {
	/**
	 * Setting ID: Site header colour.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const ID_SITE_HEADER_COLOUR = 'site_header_colour';

	/**
	 * Site header colour value: Light.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const SITE_HEADER_COLOUR_LIGHT = 'light';

	/**
	 * Site header colour value: Dark.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const SITE_HEADER_COLOUR_DARK = 'dark';

	/**
	 * Site header colour value: Stripes.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const SITE_HEADER_COLOUR_STRIPES = 'stripes';

	/**
	 * Default setting value: Site header colour.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const DEFAULT_SITE_HEADER_COLOUR = self::SITE_HEADER_COLOUR_LIGHT;

	/**
	 * Constructor. Instantiates the object.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		add_action( 'customize_register', array( $this, 'register' ) );
	}

	/**
	 * Gets site header colour class.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	public static function get_site_header_colour_class() {
		return 'site-header-' . get_theme_mod( self::ID_SITE_HEADER_COLOUR, self::DEFAULT_SITE_HEADER_COLOUR );
	}

	/**
	 * Registers customiser options.
	 *
	 * @since 1.0.0
	 *
	 * @param WP_Customize_Manager $wp_customise Theme customiser object.
	 * @return void
	 */
	public function register( $wp_customise ) {
		$wp_customise->add_setting(
			self::ID_SITE_HEADER_COLOUR,
			array(
				'default'           => self::DEFAULT_SITE_HEADER_COLOUR,
				'transport'         => 'postMessage',
				'sanitize_callback' => self::SANITIZE_CALLBACK_RADIO,
			)
		);

		$wp_customise->add_control(
			self::ID_SITE_HEADER_COLOUR,
			array(
				'priority'    => self::CONTROL_PRIORITY_APPEARANCE_SITE_HEADER_COLOUR,
				'section'     => self::SECTION_APPEARANCE,
				'label'       => esc_html__( 'Site Header Colour', 'errigal' ),
				'description' => esc_html__( 'Make sure your logo shows clearly on the colour selected.', 'errigal' ),
				'type'        => 'radio',
				'choices'     => $this->get_site_header_colours(),
			)
		);
	}

	/**
	 * Gets site header colours.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_site_header_colours() {
		return array(
			self::SITE_HEADER_COLOUR_LIGHT   => esc_html__( 'Light', 'errigal' ),
			self::SITE_HEADER_COLOUR_DARK    => esc_html__( 'Dark', 'errigal' ),
			self::SITE_HEADER_COLOUR_STRIPES => esc_html__( 'Stripes', 'errigal' ),
		);
	}
}

new Errigal_Custom_Site_Header_Colour();
