<?php
/**
 * Custom Typography Class
 *
 * @package errigal
 * @since 1.0.0
 */

/**
 * This class is in charge of typography customisation via customiser.
 *
 * @since 1.0.0
 *
 * @see Errigal_Custom_Base
 */
class Errigal_Custom_Typography extends Errigal_Custom_Base {
	/**
	 * Setting ID: Base font.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const ID_BASE_FONT = 'base_font';

	/**
	 * Setting ID: Heading font.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const ID_HEADING_FONT = 'heading_font';

	/**
	 * Setting ID: Font size.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const ID_FONT_SIZE = 'font_size';

	/**
	 * Font size value: Small.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const FONT_SIZE_SMALL = 'small-font';

	/**
	 * Font size value: Standard.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const FONT_SIZE_STANDARD = 'standard-font';

	/**
	 * Font size value: Large.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const FONT_SIZE_LARGE = 'large-font';

	/**
	 * Default setting value: Font size.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const DEFAULT_FONT_SIZE = self::FONT_SIZE_STANDARD;

	/**
	 * Editor Font size value: Small.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const EDITOR_FONT_SIZE_SMALL = '16px';

	/**
	 * Editor Font size value: Standard.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const EDITOR_FONT_SIZE_STANDARD = '18px';

	/**
	 * Editor Font size value: Large.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const EDITOR_FONT_SIZE_LARGE = '20px';

	/**
	 * Preconnect links.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	private const PRECONNECT_LINKS = array(
		'<link rel="preconnect" href="https://fonts.googleapis.com">',
		'<link rel="preconnect" href="https://fonts.gstatic.com" crossorigin>',
	);

	/**
	 * Constructor. Instantiates the object.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		add_action( 'customize_register', array( $this, 'register' ) );

		add_action( 'wp_head', array( $this, 'render_preconnect_links_in_head_tag' ), 0 );

		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_font_styles' ) );

		add_action( 'customize_preview_init', array( $this, 'declare_theme_font_array_in_customiser_preview' ) );

		add_action( 'enqueue_block_editor_assets', array( $this, 'enqueue_block_editor_font_styles' ) );

		add_action( 'after_setup_theme', array( $this, 'enqueue_classic_editor_font_style_links' ) );

		add_filter( 'tiny_mce_before_init', array( $this, 'enqueue_classic_editor_inline_font_styles' ) );
	}

	/**
	 * Registers customiser options.
	 *
	 * @since 1.0.0
	 *
	 * @param WP_Customize_Manager $wp_customise Theme customiser object.
	 * @return void
	 */
	public function register( $wp_customise ) {
		$wp_customise->add_setting(
			self::ID_BASE_FONT,
			array(
				'default'           => esc_html( $this->get_default_font_id() ),
				'transport'         => 'postMessage',
				'sanitize_callback' => self::SANITIZE_CALLBACK_SELECT,
			)
		);

		$wp_customise->add_setting(
			self::ID_HEADING_FONT,
			array(
				'default'           => esc_html( $this->get_default_font_id() ),
				'transport'         => 'postMessage',
				'sanitize_callback' => self::SANITIZE_CALLBACK_SELECT,
			)
		);

		$wp_customise->add_setting(
			self::ID_FONT_SIZE,
			array(
				'default'           => self::DEFAULT_FONT_SIZE,
				'transport'         => 'postMessage',
				'sanitize_callback' => self::SANITIZE_CALLBACK_RADIO,
			)
		);

		$wp_customise->add_control(
			self::ID_BASE_FONT,
			array(
				'section' => self::SECTION_FONT_SETTINGS,
				'label'   => esc_html__( 'Base Font', 'errigal' ),
				'type'    => 'select',
				'choices' => $this->get_font_names(),
			)
		);

		$wp_customise->add_control(
			self::ID_HEADING_FONT,
			array(
				'section' => self::SECTION_FONT_SETTINGS,
				'label'   => esc_html__( 'Heading Font', 'errigal' ),
				'type'    => 'select',
				'choices' => $this->get_font_names(),
			)
		);

		$wp_customise->add_control(
			self::ID_FONT_SIZE,
			array(
				'section' => self::SECTION_FONT_SETTINGS,
				'label'   => esc_html__( 'Font size', 'errigal' ),
				'type'    => 'radio',
				'choices' => array(
					self::FONT_SIZE_SMALL    => esc_html__( 'Small', 'errigal' ),
					self::FONT_SIZE_STANDARD => esc_html__( 'Standard', 'errigal' ),
					self::FONT_SIZE_LARGE    => esc_html__( 'Large', 'errigal' ),
				),
			)
		);
	}

	/**
	 * Gets font size class.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	public static function get_font_size_class() {
		return get_theme_mod( self::ID_FONT_SIZE, self::DEFAULT_FONT_SIZE );
	}

	/**
	 * Renders preconnect links in head tag.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function render_preconnect_links_in_head_tag() {
		foreach ( self::PRECONNECT_LINKS as $preconnect_link ) {
			echo wp_kses(
				$preconnect_link,
				array(
					'link' => array(
						'rel'         => array(),
						'href'        => array(),
						'crossorigin' => array(),
					),
				)
			);
		}
	}

	/**
	 * Enqueues font styles.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function enqueue_font_styles() {
		wp_enqueue_style(
			'errigal-base-google-font-style',
			esc_url( $this->get_font_uri( self::ID_BASE_FONT ) ),
			array(),
			esc_attr( errigal_get_stylesheet_version() )
		);

		wp_add_inline_style(
			esc_attr( errigal_get_stylesheet_name() ),
			$this->get_inline_base_font_style()
		);

		if ( $this->has_unique_heading_font() || is_customize_preview() ) {
			wp_enqueue_style(
				'errigal-heading-google-font-style',
				esc_url( $this->get_font_uri( self::ID_HEADING_FONT ) ),
				array(),
				esc_attr( errigal_get_stylesheet_version() )
			);

			wp_add_inline_style(
				esc_attr( errigal_get_stylesheet_name() ),
				$this->get_inline_heading_font_style()
			);
		}
	}

	/**
	 * Declares theme font array in customiser preview.
	 *
	 * @since 1.0.0
	 *
	 * @see errigal_customise_preview_init()
	 *
	 * @return void
	 */
	public function declare_theme_font_array_in_customiser_preview() {
		wp_localize_script(
			esc_attr( errigal_get_customise_preview_script_name() ),
			'ERRIGAL_FONT_NAMES',
			$this->get_font_names()
		);

		wp_localize_script(
			esc_attr( errigal_get_customise_preview_script_name() ),
			'ERRIGAL_FONT_FAMILIES',
			$this->get_font_families()
		);

		wp_localize_script(
			esc_attr( errigal_get_customise_preview_script_name() ),
			'ERRIGAL_FONT_SIZES',
			array(
				self::FONT_SIZE_SMALL,
				self::FONT_SIZE_STANDARD,
				self::FONT_SIZE_LARGE,
			)
		);
	}

	/**
	 * Enqueues editor font styles.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function enqueue_block_editor_font_styles() {
		$base_font_style_name = 'errigal-block-editor-base-google-font-style';

		wp_enqueue_style(
			esc_attr( $base_font_style_name ),
			esc_url( $this->get_font_uri( self::ID_BASE_FONT ) ),
			array(),
			esc_attr( errigal_get_stylesheet_version() )
		);

		wp_add_inline_style(
			esc_attr( $base_font_style_name ),
			$this->get_editor_inline_base_font_style()
		);

		if ( $this->has_unique_heading_font() ) {
			$heading_font_style_name = 'errigal-block-editor-heading-google-font-style';

			wp_enqueue_style(
				esc_attr( $heading_font_style_name ),
				esc_url( $this->get_font_uri( self::ID_HEADING_FONT ) ),
				array(),
				esc_attr( errigal_get_stylesheet_version() )
			);

			wp_add_inline_style(
				esc_attr( $heading_font_style_name ),
				$this->get_editor_inline_heading_font_style()
			);
		}
	}

	/**
	 * Enqueues font style links in classic editor.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function enqueue_classic_editor_font_style_links() {
		$base_font_url = str_replace( ',', '%2C', $this->get_font_uri( self::ID_BASE_FONT ) );

		add_editor_style( $base_font_url );

		if ( $this->has_unique_heading_font() ) {
			$heading_font_url = str_replace( ',', '%2C', $this->get_font_uri( self::ID_HEADING_FONT ) );

			add_editor_style( $heading_font_url );
		}
	}

	/**
	 * Enqueues editor font styles.
	 *
	 * @since 1.0.0
	 *
	 * @param array $settings Array with TinyMCE config.
	 * @return array
	 */
	public function enqueue_classic_editor_inline_font_styles( $settings ) {
		$settings['content_style'] = $this->get_editor_inline_base_font_style();

		if ( $this->has_unique_heading_font() ) {
			$settings['content_style'] .= $this->get_editor_inline_heading_font_style();
		}

		$settings['content_style'] = str_replace( '"', "'", $settings['content_style'] );

		return $settings;
	}

	/**
	 * Gets fonts.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	protected function get_fonts() {
		$locale = get_locale();

		if ( 'ja' === $locale ) {
			return array(
				'Noto Sans Japanese' => array(
					'Id'     => 'noto_sans_jp',
					'Name'   => 'Noto Sans JP',
					'Family' => 'Noto+Sans+JP:wght@300;400;500;700&display=swap',
				),
				'BIZ UDPGothic'      => array(
					'Id'     => 'biz_udpgothic',
					'Name'   => 'BIZ UDPGothic',
					'Family' => 'BIZ+UDPGothic:wght@400;700&display=swap',
				),
				'Sawarabi Gothic'    => array(
					'Id'     => 'sawarabi_gothic',
					'Name'   => 'Sawarabi Gothic',
					'Family' => 'Sawarabi+Gothic&display=swap',
				),
				'Kosugi Maru'        => array(
					'Id'     => 'kosugi_maru',
					'Name'   => 'Kosugi Maru',
					'Family' => 'Kosugi+Maru&display=swap',
				),
				'Kosugi'             => array(
					'Id'     => 'kosugi',
					'Name'   => 'Kosugi',
					'Family' => 'Kosugi&display=swap',
				),
			);
		}

		return array(
			'Roboto'     => array(
				'Id'     => 'roboto',
				'Name'   => 'Roboto',
				'Family' => 'Roboto:ital,wght@0,300;0,400;0,500;0,700;1,300;1,400;1,500;1,700&display=swap',
			),
			'Lato'       => array(
				'Id'     => 'lato',
				'Name'   => 'Lato',
				'Family' => 'Lato:ital,wght@0,300;0,400;0,700;1,300;1,400;1,700&display=swap',
			),
			'Montserrat' => array(
				'Id'     => 'montserrat',
				'Name'   => 'Montserrat',
				'Family' => 'Montserrat:ital,wght@0,300;0,400;0,500;0,600;0,700;1,300;1,400;1,500;1,600;1,700&display=swap',
			),
			'Raleway'    => array(
				'Id'     => 'raleway',
				'Name'   => 'Raleway',
				'Family' => 'Raleway:ital,wght@0,300;0,400;0,500;0,600;0,700;1,300;1,400;1,500;1,600;1,700&display=swap',
			),
			'Inter'      => array(
				'Id'     => 'inter',
				'Name'   => 'Inter',
				'Family' => 'Inter:wght@300;400;500;600;700&display=swap',
			),
		);
	}

	/**
	 * Gets default font ID.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	private function get_default_font_id() {
		return array_values( $this->get_fonts() )[0]['Id'];
	}

	/**
	 * Gets font names.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_font_names() {
		return array_column( $this->get_fonts(), 'Name', 'Id' );
	}

	/**
	 * Gets font families.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	private function get_font_families() {
		return array_column( $this->get_fonts(), 'Family', 'Id' );
	}

	/**
	 * Gets font URI.
	 *
	 * @since 1.0.0
	 *
	 * @param string $font_setting_id Font setting ID.
	 * @return string
	 */
	private function get_font_uri( $font_setting_id ) {
		if ( empty( $font_setting_id ) ) {
			return '';
		}

		$font_families = $this->get_font_families();

		if ( empty( $font_families ) ) {
			return '';
		}

		$font_id     = get_theme_mod( $font_setting_id, $this->get_default_font_id() );
		$font_family = array_key_exists( $font_id, $font_families ) ? $font_families[ $font_id ] : array_values( $font_families )[0];

		return sprintf(
			esc_attr( '//fonts.googleapis.com/css?family=%s' ),
			esc_attr( $font_family )
		);
	}

	/**
	 * Gets inline base font style.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	private function get_inline_base_font_style() {
		return $this->get_inline_font_style( self::ID_BASE_FONT );
	}

	/**
	 * Gets inline heading font style.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	private function get_inline_heading_font_style() {
		return $this->get_inline_font_style(
			self::ID_HEADING_FONT,
			array(
				'is_heading' => true,
			)
		);
	}

	/**
	 * Gets editor inline base font style.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	private function get_editor_inline_base_font_style() {
		return $this->get_inline_font_style(
			self::ID_BASE_FONT,
			array(
				'is_editor' => true,
			)
		);
	}

	/**
	 * Gets editor inline heading font style.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	private function get_editor_inline_heading_font_style() {
		return $this->get_inline_font_style(
			self::ID_HEADING_FONT,
			array(
				'is_editor'  => true,
				'is_heading' => true,
			)
		);
	}

	/**
	 * Gets inline font style.
	 *
	 * @since 1.0.0
	 *
	 * @param string $font_setting_id Font setting ID.
	 * @param array  $args {
	 *     Optional. Inline font style options.
	 *
	 *     @type bool $is_editor  Whether style is for editor. Default false.
	 *     @type bool $is_heading Whether style is for heading. Default false.
	 * }
	 * @return string
	 */
	private function get_inline_font_style( $font_setting_id, $args = array() ) {
		if ( empty( $font_setting_id ) ) {
			return '';
		}

		$font_names = $this->get_font_names();

		if ( empty( $font_names ) ) {
			return '';
		}

		$args = wp_parse_args(
			$args,
			array(
				'is_editor'  => false,
				'is_heading' => false,
			)
		);

		$is_heading = $args['is_heading'];

		$selectors_html           = 'html';
		$selectors_html_editor    = '.editor-styles-wrapper *, .mce-content-body';
		$selectors_heading        = 'h1,h2,h3,h3,h4,h5,h6,h1 a,h2 a,h3 a,h4 a,h5 a,h6 a, .heading-font';
		$selectors_heading_editor = sprintf( '%1$s h1,%1$s h2,%1$s h3,%1$s h3,%1$s h4,%1$s h5,%1$s h6,%1$s h1 a,%1$s h2 a,%1$s h3 a,%1$s h4 a,%1$s h5 a,%1$s h6 a,%1$s .editor-post-title__input,%2$s h1,%2$s h2,%2$s h3,%2$s h3,%2$s h4,%2$s h5,%2$s h6,%2$s h1 a,%2$s h2 a,%2$s h3 a,%2$s h4 a,%2$s h5 a,%2$s h6 a', '.editor-styles-wrapper', '.mce-content-body' );

		$font_family_format = '%s {font-family: "%s", -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, Oxygen-Sans, Ubuntu, Cantarell, "Helvetica Neue", sans-serif;}';
		$font_id            = get_theme_mod( $font_setting_id, $this->get_default_font_id() );
		$font_name          = array_key_exists( $font_id, $font_names ) ? $font_names[ $font_id ] : array_values( $font_names )[0];

		if ( $args['is_editor'] ) {
			if ( $is_heading ) {
				return sprintf( $font_family_format, esc_attr( $selectors_heading_editor ), esc_attr( $font_name ) );
			}

			$editor_body_font_size = sprintf( '.editor-styles-wrapper { font-size: %s !important; }', esc_attr( self::get_editor_body_font_size() ) );

			return sprintf(
				'%s %s',
				$editor_body_font_size,
				sprintf( $font_family_format, esc_attr( $selectors_html_editor ), esc_attr( $font_name ) )
			);
		}

		if ( $is_heading ) {
			return sprintf( $font_family_format, esc_attr( $selectors_heading ), esc_attr( $font_name ) );
		}

		return sprintf( $font_family_format, esc_attr( $selectors_html ), esc_attr( $font_name ) );
	}

	/**
	 * Determines whether unique font is set for headings.
	 *
	 * @since 1.0.0
	 *
	 * @return bool
	 */
	private function has_unique_heading_font() {
		$base_font_id    = get_theme_mod( self::ID_BASE_FONT, $this->get_default_font_id() );
		$heading_font_id = get_theme_mod( self::ID_HEADING_FONT, $this->get_default_font_id() );

		return $base_font_id !== $heading_font_id;
	}

	/**
	 * Gets editor body font size.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	private function get_editor_body_font_size() {
		$font_size_class = self::get_font_size_class();

		if ( self::FONT_SIZE_SMALL === $font_size_class ) {
			return self::EDITOR_FONT_SIZE_SMALL;
		}

		if ( self::FONT_SIZE_LARGE === $font_size_class ) {
			return self::EDITOR_FONT_SIZE_LARGE;
		}

		return self::EDITOR_FONT_SIZE_STANDARD;
	}
}

new Errigal_Custom_Typography();
