<?php
/**
 * Theme Customiser Class
 *
 * @package errigal
 * @since 1.0.0
 */

/**
 * This class is in charge of providing basic customiser options.
 *
 * @since 1.0.0
 *
 * @see Errigal_Custom_Base
 */
class Errigal_Customise extends Errigal_Custom_Base {
	/**
	 * Constructor. Instantiates the object.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		add_action( 'customize_register', array( $this, 'register' ) );
	}

	/**
	 * Registers customiser options.
	 *
	 * @since 1.0.0
	 *
	 * @param WP_Customize_Manager $wp_customise Theme customiser object.
	 * @return void
	 */
	public function register( $wp_customise ) {
		// Adds sections.
		$this->add_sections( $wp_customise );

		// Moves background color control to Appearance section.
		$wp_customise->get_control( 'background_color' )->section  = self::SECTION_APPEARANCE;
		$wp_customise->get_control( 'background_color' )->priority = self::CONTROL_PRIORITY_APPEARANCE_BACKGROUND_COLOUR;

		// Excludes pre-existing customiser settings.
		$wp_customise->remove_control( 'display_header_text' );
		$wp_customise->remove_section( 'background_image' );

		// Adds partial for blogname.
		$wp_customise->get_setting( 'blogname' )->transport = 'postMessage';
		$wp_customise->selective_refresh->add_partial(
			'blogname',
			array(
				'selector'        => '.brand-title a',
				'render_callback' => function() {
					bloginfo( 'name' );
				},
			)
		);

		// Adds partial for blogdescription.
		$wp_customise->get_setting( 'blogdescription' )->transport = 'postMessage';
		$wp_customise->selective_refresh->add_partial(
			'blogdescription',
			array(
				'selector'        => '.brand-tagline',
				'render_callback' => function() {
					bloginfo( 'description' );
				},
			)
		);

		// Adds active_callback for blogname and blogdescription to hide them when Logo is selected.
		$wp_customise->get_control( 'blogname' )->active_callback        = array( $this, 'has_no_custom_logo' );
		$wp_customise->get_control( 'blogdescription' )->active_callback = array( $this, 'has_no_custom_logo' );

		// Adds partial for custom_logo.
		$wp_customise->selective_refresh->add_partial(
			'custom_logo',
			array(
				'selector'        => '.branding',
				'render_callback' => function() {
					get_template_part( 'template-parts/header/site-header/branding' );
				},
			)
		);
	}

	/**
	 * Determines whether site has no custom logo.
	 *
	 * @since 1.0.0
	 *
	 * @return bool
	 */
	public function has_no_custom_logo() {
		return ! has_custom_logo();
	}

	/**
	 * Adds sections.
	 *
	 * @since 1.0.0
	 *
	 * @param WP_Customize_Manager $wp_customise Theme customiser object.
	 * @return void
	 */
	private function add_sections( $wp_customise ) {
		if ( empty( $wp_customise ) ) {
			return;
		}

		if ( ! is_child_theme() ) {
			$wp_customise->add_section(
				self::SECTION_PRO_FEATURES,
				array(
					'title'    => esc_html__( 'Discover errigal PRO', 'errigal' ),
					'priority' => self::SECTION_PRIORITY_PRO_FEATURES,
				)
			);
		}

		$wp_customise->add_section(
			self::SECTION_APPEARANCE,
			array(
				'title'    => esc_html__( 'Appearance', 'errigal' ),
				'priority' => self::SECTION_PRIORITY_APPEARANCE,
			)
		);

		$wp_customise->add_section(
			self::SECTION_FONT_SETTINGS,
			array(
				'title'    => esc_html__( 'Font Settings', 'errigal' ),
				'priority' => self::SECTION_PRIORITY_FONT_SETTINGS,
			)
		);

		$wp_customise->add_section(
			self::SECTION_FOOTER,
			array(
				'title' => esc_html__( 'Footer', 'errigal' ),
			)
		);

		// Moves Homepage Settings section below Appearance section.
		$wp_customise->get_section( 'static_front_page' )->priority = self::SECTION_PRIORITY_HOMEPAGE_SETTINGS;
	}
}

new Errigal_Customise();
