<?php
/**
 * Widget API: Errigal_Widget_Profile Class
 *
 * @package errigal
 * @since 1.0.3
 */

/**
 * This class is in charge of adding profile widget to theme.
 *
 * @since 1.0.3
 *
 * @see Errigal_WP_Widget
 */
class Errigal_Widget_Profile extends Errigal_WP_Widget {
	/**
	 * Base ID for widget.
	 *
	 * @since 1.0.3
	 *
	 * @var string
	 */
	private const ID_BASE = 'errigal_profile';

	/**
	 * Setting ID: Profile page ID.
	 *
	 * @since 1.0.3
	 *
	 * @var string
	 */
	private const ID_PROFILE_PAGE_ID = 'profile_page_id';

	/**
	 * Setting ID: Image URL.
	 *
	 * @since 1.0.3
	 *
	 * @var string
	 */
	private const ID_IMAGE_URL = 'image_url';

	/**
	 * Setting ID: Name.
	 *
	 * @since 1.0.3
	 *
	 * @var string
	 */
	private const ID_NAME = 'name';

	/**
	 * Setting ID: Description.
	 *
	 * @since 1.0.3
	 *
	 * @var string
	 */
	private const ID_DESCRIPTION = 'description';

	/**
	 * Setting ID: Facebook URL.
	 *
	 * @since 1.0.3
	 *
	 * @var string
	 */
	private const ID_FACEBOOK_URL = 'facebook_url';

	/**
	 * Setting ID: Twitter URL.
	 *
	 * @since 1.0.3
	 *
	 * @var string
	 */
	private const ID_TWITTER_URL = 'twitter_url';

	/**
	 * Setting ID: Instagram URL.
	 *
	 * @since 1.0.3
	 *
	 * @var string
	 */
	private const ID_INSTAGRAM_URL = 'instagram_url';

	/**
	 * Setting ID: LinkedIn URL.
	 *
	 * @since 1.0.3
	 *
	 * @var string
	 */
	private const ID_LINKEDIN_URL = 'linkedin_url';

	/**
	 * Setting ID: YouTube URL.
	 *
	 * @since 1.0.3
	 *
	 * @var string
	 */
	private const ID_YOUTUBE_URL = 'youtube_url';

	/**
	 * Setting ID: LINE URL.
	 *
	 * @since 1.0.3
	 *
	 * @var string
	 */
	private const ID_LINE_URL = 'line_url';

	/**
	 * Setting ID: GitHub URL.
	 *
	 * @since 1.0.3
	 *
	 * @var string
	 */
	private const ID_GITHUB_URL = 'github_url';

	/**
	 * Sets up new profile widget instance.
	 *
	 * @since 1.0.3
	 */
	public function __construct() {
		parent::__construct(
			self::ID_BASE,
			sprintf(
				/* translators: %s: Name of the current theme. */
				esc_html__( '(%s) Profile', 'errigal' ),
				esc_html( wp_get_theme()->get( 'Name' ) )
			),
			array(
				'classname'                   => 'errigal-profile-widget',
				'customize_selective_refresh' => true,
				'description'                 => esc_html__( 'A widget to display your profile.', 'errigal' ),
			)
		);
	}

	/**
	 * Renders HTML with content for current profile widget instance.
	 *
	 * @since 1.0.3
	 *
	 * @see WP_Widget::widget()
	 *
	 * @param array $args     Displays arguments including 'before_title', 'after_title',
	 *                        'before_widget', and 'after_widget'.
	 * @param array $instance Settings for current Archives widget instance.
	 * @return void
	 */
	public function widget( $args, $instance ) {
		$profile_page_id = $instance[ self::ID_PROFILE_PAGE_ID ] ?? '';
		$image_url       = $instance[ self::ID_IMAGE_URL ] ?? '';
		$name            = $instance[ self::ID_NAME ] ?? '';
		$description     = $instance[ self::ID_DESCRIPTION ] ?? '';
		$facebook_url    = $instance[ self::ID_FACEBOOK_URL ] ?? '';
		$twitter_url     = $instance[ self::ID_TWITTER_URL ] ?? '';
		$instagram_url   = $instance[ self::ID_INSTAGRAM_URL ] ?? '';
		$linkedin_url    = $instance[ self::ID_LINKEDIN_URL ] ?? '';
		$youtube_url     = $instance[ self::ID_YOUTUBE_URL ] ?? '';
		$line_url        = $instance[ self::ID_LINE_URL ] ?? '';
		$github_url      = $instance[ self::ID_GITHUB_URL ] ?? '';

		echo wp_kses_post( $args['before_widget'] ) .
			wp_kses_post( str_replace( 'class="', 'class="screen-reader-text ', $args['before_title'] ) ) .
			esc_html__( 'Profile', 'errigal' ) .
			wp_kses_post( $args['after_title'] );

		get_template_part(
			'template-parts/component/widget-profile',
			null,
			array(
				'profile_page_id' => $profile_page_id,
				'image_url'       => esc_url( $image_url ),
				'image_alt'       => esc_attr__( 'Profile image', 'errigal' ),
				'name'            => esc_html( $name ),
				'description'     => esc_html( $description ),
				'facebook_url'    => esc_url( $facebook_url ),
				'twitter_url'     => esc_url( $twitter_url ),
				'instagram_url'   => esc_url( $instagram_url ),
				'linkedin_url'    => esc_url( $linkedin_url ),
				'youtube_url'     => esc_url( $youtube_url ),
				'line_url'        => esc_url( $line_url ),
				'github_url'      => esc_url( $github_url ),
			)
		);

		echo wp_kses_post( $args['after_widget'] );
	}

	/**
	 * Renders HTML with settings form for profile widget.
	 *
	 * @since 1.0.3
	 *
	 * @see WP_Widget::form()
	 *
	 * @param array $instance Current settings.
	 * @return void
	 */
	public function form( $instance ) {
		$name            = $instance[ self::ID_NAME ] ?? '';
		$image_url       = $instance[ self::ID_IMAGE_URL ] ?? '';
		$profile_page_id = $instance[ self::ID_PROFILE_PAGE_ID ] ?? '';
		$description     = $instance[ self::ID_DESCRIPTION ] ?? '';
		$facebook_url    = $instance[ self::ID_FACEBOOK_URL ] ?? '';
		$twitter_url     = $instance[ self::ID_TWITTER_URL ] ?? '';
		$instagram_url   = $instance[ self::ID_INSTAGRAM_URL ] ?? '';
		$linkedin_url    = $instance[ self::ID_LINKEDIN_URL ] ?? '';
		$youtube_url     = $instance[ self::ID_YOUTUBE_URL ] ?? '';
		$line_url        = $instance[ self::ID_LINE_URL ] ?? '';
		$github_url      = $instance[ self::ID_GITHUB_URL ] ?? '';
		?>
		<div class="errigal-profile-widget">
			<?php
			$this->render_form_control(
				self::ID_NAME,
				array(
					'label' => esc_html__( 'Profile name', 'errigal' ),
					'value' => esc_attr( $name ),
				)
			);

			$this->render_form_control(
				self::ID_IMAGE_URL,
				array(
					/* translators: Must be lowercase when appropriate for the language in use. */
					'label'           => esc_html_x( 'profile image', 'Image type used as part of widget image button following Add/Remove/Replace', 'errigal' ),
					'type'            => self::FORM_CONTROL_IMAGE,
					'value'           => esc_url( $image_url ),
					'control_options' => array(
						'is_circle_image' => true,
					),
				)
			);

			$this->render_form_control(
				self::ID_PROFILE_PAGE_ID,
				array(
					'label'   => esc_html__( 'Profile page link on profile image', 'errigal' ),
					'type'    => self::FORM_CONTROL_DROPDOWN_PAGES,
					'default' => esc_html__( 'No link', 'errigal' ),
					'value'   => esc_html( $profile_page_id ),
				)
			);

			$this->render_form_control(
				self::ID_DESCRIPTION,
				array(
					'label' => esc_html__( 'Profile description (HTML tags can be used)', 'errigal' ),
					'type'  => self::FORM_CONTROL_TEXTAREA,
					'value' => esc_textarea( $description ),
				)
			);

			$this->render_form_control(
				self::ID_FACEBOOK_URL,
				array(
					'label' => esc_html__( 'Facebook URL', 'errigal' ),
					'type'  => self::FORM_CONTROL_URL,
					'value' => esc_attr( $facebook_url ),
				)
			);

			$this->render_form_control(
				self::ID_TWITTER_URL,
				array(
					'label' => esc_html__( 'Twitter URL', 'errigal' ),
					'type'  => self::FORM_CONTROL_URL,
					'value' => esc_attr( $twitter_url ),
				)
			);

			$this->render_form_control(
				self::ID_INSTAGRAM_URL,
				array(
					'label' => esc_html__( 'Instagram URL', 'errigal' ),
					'type'  => self::FORM_CONTROL_URL,
					'value' => esc_attr( $instagram_url ),
				)
			);

			$this->render_form_control(
				self::ID_LINKEDIN_URL,
				array(
					'label' => esc_html__( 'LinkedIn URL', 'errigal' ),
					'type'  => self::FORM_CONTROL_URL,
					'value' => esc_attr( $linkedin_url ),
				)
			);

			$this->render_form_control(
				self::ID_YOUTUBE_URL,
				array(
					'label' => esc_html__( 'YouTube URL', 'errigal' ),
					'type'  => self::FORM_CONTROL_URL,
					'value' => esc_attr( $youtube_url ),
				)
			);

			$this->render_form_control(
				self::ID_LINE_URL,
				array(
					'label' => esc_html__( 'LINE URL', 'errigal' ),
					'type'  => self::FORM_CONTROL_URL,
					'value' => esc_attr( $line_url ),
				)
			);

			$this->render_form_control(
				self::ID_GITHUB_URL,
				array(
					'label' => esc_html__( 'GitHub URL', 'errigal' ),
					'type'  => self::FORM_CONTROL_URL,
					'value' => esc_attr( $github_url ),
				)
			);
			?>
		</div>
		<?php
	}

	/**
	 * Handles updating settings for current profile widget instance.
	 *
	 * @since 1.0.3
	 *
	 * @see WP_Widget::update()
	 *
	 * @param array $new_instance New settings for this instance as input by the user via
	 *                            WP_Widget::form().
	 * @param array $old_instance Old settings for this instance.
	 * @return array
	 */
	public function update( $new_instance, $old_instance ) {
		$instance                             = $old_instance;
		$instance[ self::ID_NAME ]            = sanitize_text_field( $new_instance[ self::ID_NAME ] );
		$instance[ self::ID_IMAGE_URL ]       = esc_url_raw( $new_instance[ self::ID_IMAGE_URL ] );
		$instance[ self::ID_PROFILE_PAGE_ID ] = intval( $new_instance[ self::ID_PROFILE_PAGE_ID ] );
		$instance[ self::ID_DESCRIPTION ]     = wp_kses_post( $new_instance[ self::ID_DESCRIPTION ] );
		$instance[ self::ID_FACEBOOK_URL ]    = esc_url_raw( $new_instance[ self::ID_FACEBOOK_URL ] );
		$instance[ self::ID_TWITTER_URL ]     = esc_url_raw( $new_instance[ self::ID_TWITTER_URL ] );
		$instance[ self::ID_INSTAGRAM_URL ]   = esc_url_raw( $new_instance[ self::ID_INSTAGRAM_URL ] );
		$instance[ self::ID_LINKEDIN_URL ]    = esc_url_raw( $new_instance[ self::ID_LINKEDIN_URL ] );
		$instance[ self::ID_YOUTUBE_URL ]     = esc_url_raw( $new_instance[ self::ID_YOUTUBE_URL ] );
		$instance[ self::ID_LINE_URL ]        = esc_url_raw( $new_instance[ self::ID_LINE_URL ] );
		$instance[ self::ID_GITHUB_URL ]      = esc_url_raw( $new_instance[ self::ID_GITHUB_URL ] );

		return $instance;
	}
}

/**
 * Register profile widget.
 *
 * @since 1.0.3
 *
 * @link https://developer.wordpress.org/themes/functionality/widgets/
 *
 * @return void
 */
function errigal_register_profile_widget() {
	register_widget( 'Errigal_Widget_Profile' );
}
add_action( 'widgets_init', 'errigal_register_profile_widget' );
