<?php
/**
 * Widget API: Errigal_WP_Widget base class
 *
 * @package errigal
 * @since 1.0.3
 */

/**
 * This class is in charge of extending core widget base class.
 *
 * @since 1.0.3
 *
 * @see WP_Widget
 */
class Errigal_WP_Widget extends WP_Widget {
	/**
	 * Form control: Dropdown pages.
	 *
	 * @since 1.0.3
	 *
	 * @var string
	 */
	protected const FORM_CONTROL_DROPDOWN_PAGES = 'dropdown-pages';

	/**
	 * Form control: Image.
	 *
	 * @since 1.0.3
	 *
	 * @var string
	 */
	protected const FORM_CONTROL_IMAGE = 'image';

	/**
	 * Form control: Text.
	 *
	 * @since 1.0.3
	 *
	 * @var string
	 */
	protected const FORM_CONTROL_TEXT = 'text';

	/**
	 * Form control: Textarea.
	 *
	 * @since 1.0.3
	 *
	 * @var string
	 */
	protected const FORM_CONTROL_TEXTAREA = 'textarea';

	/**
	 * Form control: URL.
	 *
	 * @since 1.0.3
	 *
	 * @var string
	 */
	protected const FORM_CONTROL_URL = 'url';

	/**
	 * Renders form control.
	 *
	 * @since 1.0.3
	 *
	 * @param string $id   Form control ID.
	 * @param array  $args {
	 *     Optional. Array of properties for new control object. Default empty array.
	 *
	 *     @type string $label           Label for control. Default empty.
	 *     @type string $type            Control type. Core controls include 'dropdown-pages', 'image', 'text', 'textarea'.
	 *                                   Additional input types such as 'url' are supported implicitly. Default 'text'.
	 *     @type string $default         Default value control. Default empty.
	 *     @type string $value           Value for control. Default empty.
	 *     @type array  $control_options {
	 *         @type bool   $is_circle_image Indicates if image on control shows in circle. This is for 'image'. Default false.
	 *     }
	 * }
	 * @return void
	 */
	protected function render_form_control( $id, $args = array() ) {
		if ( empty( $id ) ) {
			return;
		}

		$args = wp_parse_args(
			$args,
			array(
				'label'           => '',
				'type'            => self::FORM_CONTROL_TEXT,
				'default'         => '',
				'value'           => '',
				'control_options' => array(
					'is_circle_image' => false,
				),
			)
		);

		$label           = $args['label'];
		$type            = $args['type'];
		$default         = $args['default'];
		$value           = $args['value'];
		$is_circle_image = $args['control_options']['is_circle_image'];

		if ( empty( $label ) || empty( $type ) ) {
			return;
		}

		if ( self::FORM_CONTROL_DROPDOWN_PAGES === $type ) {
			?>
			<p>
				<?php $this->render_form_label( $id, $label ); ?>
				<select class="errigal-widget__select" id="<?php echo esc_attr( $this->get_field_id( $id ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( $id ) ); ?>">
					<option value="0"><?php echo esc_html( $default ); ?></option>
					<?php foreach ( get_pages() as $page ) : ?>
						<option value="<?php echo esc_attr( $page->ID ); ?>" <?php selected( $value, $page->ID ); ?>><?php echo esc_html( $page->post_title ); ?></option>
					<?php endforeach; ?>
				</select>
			</p>
			<?php

			return;
		}

		if ( self::FORM_CONTROL_IMAGE === $type ) {
			$image_container_class = 'errigal-widget__image-container';

			if ( true === $is_circle_image ) {
				$image_container_class .= ' circle-image';
			}

			?>
			<div class="errigal-widget__image-control">
				<div class="<?php echo esc_attr( $image_container_class ); ?>" <?php echo empty( $value ) ? 'style="display: none;"' : ''; ?>>
					<?php
					get_template_part(
						'template-parts/component/widget/image',
						null,
						array(
							'image_class' => 'errigal-widget__image',
							'image_url'   => $value,
						)
					);
					?>
					<input
						class="widefat errigal-widget__image-path"
						id="<?php echo esc_attr( $this->get_field_id( $id ) ); ?>"
						name="<?php echo esc_attr( $this->get_field_name( $id ) ); ?>"
						type="url" value="<?php echo esc_url( $value ); ?>" />
				</div>
				<div class="errigal-widget__add-image-button-container" <?php echo ! empty( $value ) ? 'style="display: none;"' : ''; ?>>
					<button class="errigal-widget__add-image-button" type="button"><?php printf( /* translators: %s: Image type. */ esc_html__( 'Add %s', 'errigal' ), esc_html( $label ) ); ?>
					</button>
				</div>
				<div class="errigal-widget__image-buttons-container" <?php echo empty( $value ) ? 'style="display: none;"' : ''; ?>>
					<button class="button errigal-widget__replace-image-button" type="button"><?php printf( /* translators: %s: Image type. */ esc_html__( 'Replace %s', 'errigal' ), esc_html( $label ) ); ?></button>
					<button class="button errigal-widget__remove-image-button" type="button"><?php printf( /* translators: %s: Image type. */ esc_html__( 'Remove %s', 'errigal' ), esc_html( $label ) ); ?></button>
				</div>
			</div>
			<?php

			return;
		}

		if ( self::FORM_CONTROL_TEXT === $type || self::FORM_CONTROL_URL === $type ) {
			?>
			<p>
				<?php $this->render_form_label( $id, $label ); ?>
				<input
					class="widefat errigal-widget__text"
					id="<?php echo esc_attr( $this->get_field_id( $id ) ); ?>"
					name="<?php echo esc_attr( $this->get_field_name( $id ) ); ?>"
					type="<?php echo esc_attr( $type ); ?>"
					value="<?php echo esc_attr( $value ); ?>" />
			</p>
			<?php

			return;
		}

		if ( self::FORM_CONTROL_TEXTAREA === $type ) {
			?>
			<p>
				<?php $this->render_form_label( $id, $label ); ?>
				<textarea 
					class="widefat errigal-widget__textarea"
					rows="6"
					cols="20"
					id="<?php echo esc_attr( $this->get_field_id( $id ) ); ?>"
					name="<?php echo esc_attr( $this->get_field_name( $id ) ); ?>"
					type="text"><?php echo esc_textarea( $value ); ?></textarea>
			</p>
			<?php
		}
	}

	/**
	 * Renders form label.
	 *
	 * @since 1.0.3
	 *
	 * @param string $id    Form control ID.
	 * @param array  $label Label for control.
	 * @return void
	 */
	private function render_form_label( $id, $label ) {
		if ( empty( $id ) || empty( $label ) ) {
			return;
		}

		?>
		<label for="<?php echo esc_attr( $this->get_field_id( $id ) ); ?>"><?php echo esc_html( $label . ':' ); ?></label>
		<?php
	}
}
