<?php
/**
 * Functions and definitions
 *
 * @link https://developer.wordpress.org/themes/basics/theme-functions/
 *
 * @package errigal
 * @since 1.0.0
 */

if ( ! function_exists( 'errigal_setup' ) ) :
	/**
	 * Sets up theme defaults and registers support for various WordPress features.
	 *
	 * Note that this function is hooked into after_setup_theme hook, which
	 * runs before init hook. The init hook is too late for some features, such
	 * as indicating support for post thumbnails.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	function errigal_setup() {
		/*
		 * Makes theme available for translation.
		 * Translations can be filed in /languages/ directory.
		 * If you're building a theme based on errigal, use find and replace
		 * to change 'errigal' to the name of your theme in all template files.
		 */
		load_theme_textdomain( 'errigal', get_template_directory() . '/languages' );

		// Adds default posts and comments RSS feed links to head.
		add_theme_support( 'automatic-feed-links' );

		/*
		 * Lets WordPress manage document title.
		 * By adding theme support, we declare that this theme does not use a
		 * hard-coded <title> tag in document head, and expect WordPress to
		 * provide it for us.
		 */
		add_theme_support( 'title-tag' );

		/*
		 * Adds post-formats support.
		 *
		 * @link https://developer.wordpress.org/themes/functionality/post-formats/
		 */
		add_theme_support(
			'post-formats',
			array(
				'aside',
				'gallery',
				'link',
				'image',
				'quote',
				'status',
				'video',
				'audio',
				'chat',
			)
		);

		/*
		 * Enables support for post thumbnails on posts and pages.
		 *
		 * @link https://developer.wordpress.org/themes/functionality/featured-images-post-thumbnails/
		 */
		add_theme_support( 'post-thumbnails' );

		// Registers main menu location.
		register_nav_menus(
			array(
				errigal_get_main_menu_location() => esc_html__( 'Main Menu', 'errigal' ),
			)
		);

		/*
		 * Switches default core markup for search form, comment form, and comments
		 * to output valid HTML5.
		 *
		 * @link https://developer.wordpress.org/reference/functions/add_theme_support/#html5
		 * @link https://make.wordpress.org/themes/handbook/review/accessibility/required/
		 */
		add_theme_support(
			'html5',
			array(
				'search-form',
				'comment-form',
				'comment-list',
				'gallery',
				'caption',
				'style',
				'script',
				'navigation-widgets',
			)
		);

		// Sets up the WordPress core custom background feature.
		add_theme_support(
			'custom-background',
			apply_filters(
				'errigal_custom_background_args',
				array(
					'default-color' => 'ffffff',
					'default-image' => '',
				)
			)
		);

		// Adds theme support for selective refresh for widgets.
		add_theme_support( 'customize-selective-refresh-widgets' );

		/**
		 * Adds support for core custom logo.
		 *
		 * @link https://codex.wordpress.org/Theme_Logo
		 */
		$custom_logo_height = 140;
		$custom_logo_width  = 250;

		add_theme_support(
			'custom-logo',
			array(
				'height'      => $custom_logo_height,
				'width'       => $custom_logo_width,
				'flex-width'  => true,
				'flex-height' => true,
			)
		);

		// Adds theme support for selective refresh for widgets.
		add_theme_support( 'customize-selective-refresh-widgets' );

		// Adds support for Block Styles.
		add_theme_support( 'wp-block-styles' );

		// Adds support for full and wide align images.
		add_theme_support( 'align-wide' );

		// Adds support for responsive embedded content.
		add_theme_support( 'responsive-embeds' );
	}
endif;
add_action( 'after_setup_theme', 'errigal_setup' );

/**
 * Sets content width in pixels, based on theme design and stylesheet.
 *
 * Priority 0 to make it available to lower priority callbacks.
 *
 * @since 1.0.0
 *
 * @global int $content_width
 *
 * @return void
 */
function errigal_content_width() {
	// This variable is intended to be overruled from themes.
	// Open WPCS issue: {@link https://github.com/WordPress-Coding-Standards/WordPress-Coding-Standards/issues/1043}.
	// phpcs:ignore WordPress.NamingConventions.PrefixAllGlobals.NonPrefixedVariableFound
	$GLOBALS['content_width'] = apply_filters( 'errigal_content_width', 640 );
}
add_action( 'after_setup_theme', 'errigal_content_width', 0 );

/**
 * Enqueues script for customiser preview.
 *
 * @since 1.0.0
 *
 * @see Errigal_Custom_Typography::declare_theme_font_array_in_customiser_preview()
 *
 * @return void
 */
function errigal_customise_preview_init() {
	wp_enqueue_script(
		esc_attr( errigal_get_customise_preview_script_name() ),
		esc_url( errigal_get_script_uri( 'customise-preview.min.js' ) ),
		array( 'customize-preview' ),
		esc_attr( errigal_get_script_version() ),
		true
	);
}
add_action( 'customize_preview_init', 'errigal_customise_preview_init' );

/**
 * Enqueues script for customiser preview controls.
 *
 * @since 1.0.0
 *
 * @link https://developer.wordpress.org/reference/hooks/customize_controls_enqueue_scripts/
 *
 * @return void
 */
function errigal_customise_controls_enqueue_scripts() {
	wp_enqueue_script(
		esc_attr( errigal_get_customise_preview_script_name() . '-controls' ),
		esc_url( errigal_get_script_uri( 'customise-preview-controls.min.js' ) ),
		array( 'jquery', 'customize-controls' ),
		esc_attr( errigal_get_script_version() ),
		true
	);
}
add_action( 'customize_controls_enqueue_scripts', 'errigal_customise_controls_enqueue_scripts' );

/**
 * Enqueues scripts.
 *
 * @since 1.0.0
 *
 * @global bool $is_IE Whether browser is Internet Explorer.
 *
 * @return void
 */
function errigal_enqueue_scripts() {
	global $is_IE;

	if ( errigal_has_nav_menu( errigal_get_main_menu_location() ) ) {
		if ( wp_is_mobile() ) {
			// Enqueues desktop menu script.
			wp_enqueue_script(
				'errigal-mobile-menu',
				esc_url( errigal_get_script_uri( 'mobile-menu.min.js' ) ),
				array(),
				esc_attr( errigal_get_script_version() ),
				true
			);
		} else {
			// Enqueues mobile menu script.
			wp_enqueue_script(
				'errigal-desktop-menu',
				esc_url( errigal_get_script_uri( 'desktop-menu.min.js' ) ),
				array(),
				esc_attr( errigal_get_script_version() ),
				true
			);
		}
	}

	if ( wp_is_mobile() ) {
		// Enqueues mobile script.
		wp_enqueue_script(
			'errigal-mobile',
			esc_url( errigal_get_script_uri( 'mobile.min.js' ) ),
			array(),
			esc_attr( errigal_get_script_version() ),
			true
		);
	} else {
		// Enqueues desktop script.
		wp_enqueue_script(
			'errigal-desktop',
			esc_url( errigal_get_script_uri( 'desktop.min.js' ) ),
			array(),
			esc_attr( errigal_get_script_version() ),
			true
		);
	}

	if ( ( is_singular( 'post' ) || is_page() || is_attachment() ) && ( comments_open() || '0' !== get_comments_number() ) ) {
		$comments_script = 'errigal-comments';

		// Enqueues comments script.
		wp_enqueue_script(
			esc_attr( $comments_script ),
			esc_url( errigal_get_script_uri( 'comments.min.js' ) ),
			array(),
			esc_attr( errigal_get_script_version() ),
			true
		);

		// // Creates comment form error array.
		wp_localize_script(
			esc_attr( $comments_script ),
			'ERRIGAL_COMMENT_FORM_ERROR',
			array(
				'CommentRequired'      => esc_html__( 'Please enter your comment.', 'errigal' ),
				'AuthorRequired'       => esc_html__( 'Please enter your name.', 'errigal' ),
				'EmailAddressRequired' => esc_html__( 'Please enter your email address.', 'errigal' ),
				'EmailAddressInvalid'  => esc_html__( 'Please enter a valid email address.', 'errigal' ),
			)
		);
	}

	if ( is_singular() && comments_open() && get_option( 'thread_comments' ) ) {
		// Enqueues comment reply script.
		wp_enqueue_script( 'comment-reply' );
	}

	if ( is_user_logged_in() || is_admin_bar_showing() ) {
		// Enqueues logged-in script.
		wp_enqueue_script(
			'errigal-admin-bar-showing',
			esc_url( errigal_get_script_uri( 'admin-bar-showing.min.js' ) ),
			array(),
			esc_attr( errigal_get_script_version() ),
			true
		);
	}

	if ( $is_IE ) {
		// Enqueues IE script.
		wp_enqueue_script(
			'errigal-ie',
			esc_url( get_template_directory_uri() . '/assets/js/ie.min.js' ),
			array(),
			esc_attr( errigal_get_script_version() ),
			true
		);
	}
}
add_action( 'wp_enqueue_scripts', 'errigal_enqueue_scripts' );

/**
 * Enqueues block editor script.
 *
 * @since 1.0.0
 *
 * @return void
 */
function errigal_block_editor_script() {
	// Enqueues editor script.
	wp_enqueue_script(
		'errigal-editor',
		esc_url( errigal_get_script_uri( 'editor.min.js' ) ),
		array( 'wp-blocks', 'wp-dom' ),
		esc_attr( errigal_get_script_version() ),
		true
	);
}
add_action( 'enqueue_block_editor_assets', 'errigal_block_editor_script' );

/**
 * Register widget area.
 *
 * @since 1.0.0
 *
 * @link https://developer.wordpress.org/themes/functionality/sidebars/#registering-a-sidebar
 *
 * @return void
 */
function errigal_widgets_init() {
	register_sidebar(
		array(
			'name'          => esc_html__( 'Sidebar', 'errigal' ),
			'id'            => errigal_get_sidebar_id(),
			'description'   => esc_html__( 'Add widgets here.', 'errigal' ),
			'before_widget' => '<section id="%1$s" class="widget %2$s">',
			'after_widget'  => '</section>',
			'before_title'  => '<h2 class="widget-title">',
			'after_title'   => '</h2>',
		)
	);
}
add_action( 'widgets_init', 'errigal_widgets_init' );

// Admin.
require get_template_directory() . '/classes/class-errigal-admin.php';

// Shared: SVG icons.
require get_template_directory() . '/classes/class-errigal-svg-icons.php';

// Shared: Social links.
require get_template_directory() . '/classes/class-errigal-social-links.php';

// Base class: Customiser.
require get_template_directory() . '/classes/class-errigal-custom-base.php';

// Base class: Widget.
require get_template_directory() . '/classes/class-errigal-wp-widget.php';

// Custom template tags for this theme.
require get_template_directory() . '/inc/template-tags.php';

// Functions which enhance the theme by hooking into WordPress.
require get_template_directory() . '/inc/template-functions.php';

// Customiser: Default.
require get_template_directory() . '/classes/class-errigal-customise.php';

// Customiser: Pro Features: Pro features.
if ( ! is_child_theme() ) {
	// Pro features control.
	require get_template_directory() . '/classes/class-errigal-customise-pro-features-control.php';

	// Pro Features.
	require get_template_directory() . '/classes/class-errigal-custom-pro-features.php';
}

// Customiser: Appearance: Custom colour scheme.
require get_template_directory() . '/classes/class-errigal-custom-colour-scheme.php';

// Customiser: Appearance: Custom site header Colour.
require get_template_directory() . '/classes/class-errigal-custom-site-header-colour.php';

// Customiser: Font Styles: Custom typography.
require get_template_directory() . '/classes/class-errigal-custom-typography.php';

// Customiser: Footer: Custom footer.
require get_template_directory() . '/classes/class-errigal-custom-footer.php';

// Widget: Profile.
require get_template_directory() . '/classes/class-errigal-widget-profile.php';
