<?php
/**
 * Functions which enhance the theme by hooking into WordPress
 *
 * @package errigal
 * @since 1.0.0
 */

/**
 * Adds custom body classes.
 *
 * @since 1.0.0
 *
 * @link https://developer.wordpress.org/reference/hooks/body_class/
 *
 * @param array $classes Array of body class names.
 * @return array
 */
function errigal_body_classes( $classes ) {
	// Adds 'hfeed' to non-singular pages.
	if ( ! is_singular() ) {
		$classes[] = 'hfeed';
	}

	// Adds 'no-navigation-menu' when there is no navigation menu.
	if ( ! errigal_has_nav_menu( errigal_get_main_menu_location() ) ) {
		$classes[] = 'no-navigation-menu';
	}

	// Adds 'mobile-device' when page is viewed on mobile device.
	if ( wp_is_mobile() ) {
		$classes[] = 'mobile-device';
	}

	// Adds 'no-sidebar' when page shows no sidebar.
	if ( ! errigal_can_show_sidebar() || errigal_is_no_sidebar_post() ) {
		$classes[] = 'no-sidebar';
	}

	return $classes;
}
add_filter( 'body_class', 'errigal_body_classes' );

/**
 * Adds custom classes to comment form input fields.
 *
 * @since 1.0.0
 *
 * @see errigal_comment_form_textarea()
 * @link https://developer.wordpress.org/reference/hooks/comment_form_default_fields/
 *
 * @param string[] $fields Array of default comment fields.
 * @return string[]
 */
function errigal_comment_form_input_fields( $fields ) {
	foreach ( $fields as &$field ) {
		if ( false === strpos( $field, 'comment-form-cookies-consent' ) ) {
			$field = str_replace( 'class="', 'class="input-field ', $field );
		} else {
			$field = str_replace( 'class="', 'class="checkbox ', $field );
		}
	}

	return $fields;
}
add_filter( 'comment_form_default_fields', 'errigal_comment_form_input_fields' );

/**
 * Adds custom class to comment form textarea.
 *
 * @since 1.0.0
 *
 * @see errigal_comment_form_input_fields()
 * @link https://developer.wordpress.org/reference/hooks/comment_form_defaults/
 *
 * @param array $defaults Default comment form arguments.
 * @return array
 */
function errigal_comment_form_textarea( $defaults ) {
	$defaults['comment_field'] = str_replace( 'class="', 'class="input-field ', $defaults['comment_field'] );

	return $defaults;
}
add_filter( 'comment_form_defaults', 'errigal_comment_form_textarea' );

/**
 * Adds custom classes to comment reply link.
 *
 * @since 1.0.0
 *
 * @link https://developer.wordpress.org/reference/hooks/comment_reply_link/
 *
 * @param array $link HTML markup for comment reply link.
 * @return array
 */
function errigal_comment_reply_link( $link ) {
	$link = str_replace( "class='comment-reply-link", "class='button outlined comment-reply-link", $link );

	return $link;
}
add_filter( 'comment_reply_link', 'errigal_comment_reply_link' );

/**
 * Adds custom class to custom logo.
 *
 * @since 1.0.0
 *
 * @link https://developer.wordpress.org/reference/hooks/get_custom_logo/
 *
 * @param string $html Custom logo HTML output.
 * @return string
 */
function errigal_custom_logo( $html ) {
	$html = str_replace( 'custom-logo', 'custom-logo brand-logo', $html );

	return $html;
}
add_filter( 'get_custom_logo', 'errigal_custom_logo' );

/**
 * Adds pingback url auto-discovery header for single posts, pages, or attachments.
 *
 * @since 1.0.0
 *
 * @link https://developer.wordpress.org/reference/hooks/wp_head/
 *
 * @return void
 */
function errigal_pingback_header() {
	if ( is_singular() && pings_open() ) {
		echo '<link rel="pingback" href="', esc_url( get_bloginfo( 'pingback_url' ) ), '">';
	}
}
add_action( 'wp_head', 'errigal_pingback_header' );

/**
 * Adds custom classes to tag.
 *
 * @since 1.0.0
 *
 * @link https://developer.wordpress.org/reference/hooks/term_links-taxonomy/
 *
 * @param string $links Array of term links.
 * @return string
 */
function errigal_post_tag( $links ) {
	return str_replace( '<a', '<a class="chip secondary"', $links );
}
add_filter( 'term_links-post_tag', 'errigal_post_tag' );

/**
 * Gets protected post password form.
 *
 * @since 1.0.0
 * @since 1.0.3 Corrected callback function parameters
 *
 * @link https://developer.wordpress.org/reference/hooks/the_password_form/
 *
 * @param string      $output The password form HTML output.
 * @param int|WP_Post $post   Optional. Post ID or WP_Post object. Default is global $post.
 * @return string
 */
function errigal_protected_post_password_form( $output, $post = 0 ) {
	$post   = get_post( $post );
	$label  = 'pwbox-' . ( empty( $post->ID ) ? wp_rand() : $post->ID );
	$output = '<p>' . esc_html__( 'This content is password protected. To view it please enter your password below.', 'errigal' ) . '</p>' .
		'<form action="' . esc_url( site_url( 'wp-login.php?action=postpass', 'login_post' ) ) . '" class="post-password-form" method="post">' .
			'<p class="input-field">' .
				'<label for="' . esc_attr( $label ) . '">' . esc_html_x( 'Password', 'Post password form', 'errigal' ) . '</label>' .
				'<input required name="post_password" id="' . esc_attr( $label ) . '" type="password" size="20" />' .
			'</p>' .
			'<input type="submit" class="post-password-form__submit button outlined" name="' . esc_attr_x( 'Submit', 'Post password form', 'errigal' ) . '" value="' . esc_attr_x( 'Enter', 'Post password form', 'errigal' ) . '" />' .
		'</form>';

	return $output;
}
add_filter( 'the_password_form', 'errigal_protected_post_password_form', 10, 2 );

/**
 * Adds custom class to each category in category list.
 *
 * @since 1.0.0
 *
 * @link https://developer.wordpress.org/reference/hooks/the_category/
 *
 * @param string $thelist   List of categories for current post.
 * @param string $separator Separator used between categories.
 * @param string $parents   How to display category parents. Accepts 'multiple',
 *                          'single', or empty.
 * @return string
 */
function errigal_the_category( $thelist, $separator, $parents ) {
	return str_replace( '<a', '<a class="chip"', $thelist );
}
add_filter( 'the_category', 'errigal_the_category', 10, 3 );

/**
 * Adds fallback title if post has no title given.
 *
 * @since 1.0.0
 *
 * @link https://developer.wordpress.org/reference/hooks/the_title/
 *
 * @param string $title Post title.
 * @return string
 */
function errigal_the_title( $title ) {
	return '' === $title ? esc_html_x( 'Untitled', 'Fallback for missing titles', 'errigal' ) : $title;
}
add_filter( 'the_title', 'errigal_the_title' );

/**
 * Adds down arrow to menu item with sub menu.
 *
 * @since 1.0.1
 *
 * @link https://developer.wordpress.org/reference/hooks/walker_nav_menu_start_el/
 *
 * @param string $output Menu item's starting HTML output.
 * @param object $item   Menu item data object.
 * @param int    $depth  Depth of menu item.
 * @param object $args   Object of wp_nav_menu() arguments.
 * @return string
 */
function errigal_add_sub_menu_down_arrow( $output, $item, $depth, $args ) {
	if ( in_array( 'menu-item-has-children', $item->classes, true ) ) {

		// Add toggle button.
		$output  = str_replace( '</a>', '<span class="screen-reader-text">' . esc_html__( '. Sub menu is available. Go to the button next to this link to open sub menu.', 'errigal' ) . '</span></a>', $output );
		$output .= '<button class="sub-menu-toggle" aria-expanded="false">';
		$output .= wp_kses( errigal_get_icon_svg( 'down-arrow' ), errigal_get_allowed_html_svg() );
		$output .= '<span class="sub-menu-toggle-text-open screen-reader-text" aria-hidden="false">' . esc_html__( 'Open menu', 'errigal' ) . '</span>';
		$output .= '<span class="sub-menu-toggle-text-close screen-reader-text" aria-hidden="true">' . esc_html__( 'Close menu', 'errigal' ) . '</span>';
		$output .= '</button>';
	}
	return $output;
}
add_filter( 'walker_nav_menu_start_el', 'errigal_add_sub_menu_down_arrow', 10, 4 );

/**
 * Filters arguments used in retrieving page links for paginated posts.
 *
 * @since 1.0.0
 *
 * @link https://developer.wordpress.org/reference/hooks/wp_link_pages_args/
 * @global int  $page      Page number of single post.
 * @global int  $numpages  Number of pages in post.
 * @global bool $multipage Whether current post has multiple pages.
 * @global bool $more      Whether WordPress should enforce <!--more--> tag for current post.
 *
 * @param array $parsed_args Array of page link arguments. See wp_link_pages()
 *                           for information on accepted arguments.
 * @return array
 */
function errigal_wp_link_pages_args( $parsed_args ) {
	global $page, $numpages, $multipage, $more;

	if ( 'next_and_number' === $parsed_args['next_or_number'] ) {
		$parsed_args['next_or_number'] = 'number';
		$prev                          = '';
		$next                          = '';

		if ( $multipage ) {
			if ( $more ) {
				$i = $page - 1;

				if ( $i && $more ) {
					$prev .= str_replace( 'post-page-numbers', 'prev post-page-numbers', _wp_link_page( $i ) );
					$prev .= $parsed_args['link_before'] . $parsed_args['previouspagelink'] . $parsed_args['link_after'] . '</a>';
				}

				$i = $page + 1;

				if ( $i <= $numpages && $more ) {
					$next .= str_replace( 'post-page-numbers', 'next post-page-numbers', _wp_link_page( $i ) );
					$next .= $parsed_args['link_before'] . $parsed_args['nextpagelink'] . $parsed_args['link_after'] . '</a>';
				}
			}
		}

		$parsed_args['before'] = $parsed_args['before'] . $prev;
		$parsed_args['after']  = $next . $parsed_args['after'];
	}
	return $parsed_args;
}
add_filter( 'wp_link_pages_args', 'errigal_wp_link_pages_args' );

if ( ! function_exists( 'errigal_can_show_sidebar' ) ) {
	/**
	 * Determines whether sidebar can be displayed.
	 *
	 * @since 1.0.0
	 *
	 * @return bool
	 */
	function errigal_can_show_sidebar() {
		return errigal_has_active_sidebar();
	}
}

/**
 * Gets allowed HTML for post and SVG.
 *
 * @since 1.0.0
 *
 * @return array
 */
function errigal_get_allowed_html_post_svg() {
	return array_merge(
		wp_kses_allowed_html( 'post' ),
		errigal_get_allowed_html_svg()
	);
}

if ( ! function_exists( 'errigal_get_allowed_html_svg' ) ) {
	/**
	 * Gets allowed HTML for SVG.
	 *
	 * @since 1.0.0
	 *
	 * @see Errigal_SVG_Icons::get_allowed_html_svg()
	 *
	 * @return array
	 */
	function errigal_get_allowed_html_svg() {
		return Errigal_SVG_Icons::get_allowed_html_svg();
	}
}

/**
 * Gets continue reading text.
 *
 * @since 1.0.0
 *
 * @return string
 */
function errigal_get_continue_reading_text() {
	return sprintf(
		/* translators: %s: Name of current post. */
		esc_html__( 'Continue reading %s', 'errigal' ),
		the_title( '<span class="screen-reader-text">', '</span>', false )
	);
}

/**
 * Gets name of customise preview script.
 *
 * @since 1.0.0
 *
 * @return string
 */
function errigal_get_customise_preview_script_name() {
	return 'errigal-customise-preview';
}

if ( ! function_exists( 'errigal_get_icon_svg' ) ) {
	/**
	 * Gets SVG code for given icon.
	 *
	 * @since 1.0.0
	 *
	 * @see Errigal_SVG_Icons::get_svg()
	 *
	 * @param string $icon Icon.
	 * @param string $size Icon size of sm, base, md, lg, xl, 2xl or 3xl. Default base.
	 * @return string
	 */
	function errigal_get_icon_svg( $icon, $size = 'base' ) {
		return Errigal_SVG_Icons::get_svg( $icon, $size );
	}
}

/**
 * Gets main menu location.
 *
 * @since 1.0.0
 *
 * @return string
 */
function errigal_get_main_menu_location() {
	return 'main-menu';
}

/**
 * Gets script URI.
 *
 * @since 1.0.4
 *
 * @global bool $is_IE Whether browser is Internet Explorer.
 *
 * @param string $file_name Name of JavaScript script file.
 * @return string
 */
function errigal_get_script_uri( $file_name ) {
	global $is_IE;

	$is_js_file = preg_match( '/^[a-zA-Z0-9-]+(\.[a-zA-Z0-9-]+)*\.js$/i', $file_name );

	if ( ! $is_js_file ) {
			return '';
	}

	return get_template_directory_uri() . '/assets/js/' . ( $is_IE ? 'ie-' : '' ) . $file_name;
}

/**
 * Gets script version.
 *
 * @since 1.0.0
 *
 * @return string
 */
function errigal_get_script_version() {
	return errigal_get_theme_version();
}

/**
 * Gets sidebar ID.
 *
 * @since 1.0.0
 *
 * @return string
 */
function errigal_get_sidebar_id() {
	return 'sidebar-1';
}

if ( ! function_exists( 'errigal_get_social_links' ) ) {
	/**
	 * Gets social links.
	 *
	 * @since 1.0.3
	 *
	 * @see Errigal_Social_Links::get_social_links()
	 *
	 * @param array $args {
	 *     Optional. Social link options.
	 *
	 *     @type string $icon_size     Icon-size of sm, base, md, lg, xl, 2xl or 3xl. Default md.
	 *     @type string $facebook_url  Facebook URL.
	 *     @type string $twitter_url   Twitter URL.
	 *     @type string $instagram_url Instagram URL.
	 *     @type string $linkedin_url  LinkedIn URL.
	 *     @type string $youtube_url   YouTube URL.
	 *     @type string $line_url      LINE URL.
	 *     @type string $github_url    GitHub URL.
	 * @return string
	 */
	function errigal_get_social_links( $args = array() ) {
		return Errigal_Social_Links::get_social_links( $args );
	}
}

/**
 * Gets name of the stylesheet.
 *
 * @since 1.0.0
 *
 * @return string
 */
function errigal_get_stylesheet_name() {
	return 'errigal-style';
}

if ( ! function_exists( 'errigal_get_stylesheet_version' ) ) {
	/**
	 * Gets stylesheet version.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	function errigal_get_stylesheet_version() {
		return errigal_get_theme_version();
	}
}

/**
 * Gets theme version.
 *
 * @since 1.0.0
 *
 * @return string
 */
function errigal_get_theme_version() {
	$parent = wp_get_theme()->parent();
	$theme  = empty( $parent ) ? wp_get_theme() : $parent;

	return esc_html( $theme->get( 'Version' ) );
}

/**
 * Determines whether sidebar contains widgets.
 *
 * @since 1.0.0
 *
 * @return bool
 */
function errigal_has_active_sidebar() {
	return is_active_sidebar( errigal_get_sidebar_id() );
}

/**
 * Determines whether registered nav menu location has menu assigned to it
 * and menu is not empty.
 *
 * @since 1.0.0
 *
 * @param string $location Menu location identifier.
 * @return bool
 */
function errigal_has_nav_menu( $location = null ) {
	return apply_filters(
		'errigal_has_nav_menu',
		has_nav_menu( $location ) &&
		wp_nav_menu(
			array(
				'theme_location' => $location,
				'echo'           => false,
			)
		) !== false
	);
}

if ( ! function_exists( 'errigal_is_no_sidebar_post' ) ) {
	/**
	 * Determines whether page is in no sidebar template.
	 *
	 * @since 1.0.0
	 *
	 * @return bool
	 */
	function errigal_is_no_sidebar_post() {
		return is_page_template( 'templates/no-sidebar-page.php' ) || is_page_template( 'templates/no-sidebar-post.php' );
	}
}

/**
 * Renders SVG code for given icon.
 *
 * @since 1.0.0
 *
 * @param string $icon Icon.
 * @param string $size Icon-size of sm, base, md, lg or xl. Default base.
 * @return void
 */
function errigal_render_icon_svg( $icon, $size = 'base' ) {
	echo wp_kses( errigal_get_icon_svg( $icon, $size ), errigal_get_allowed_html_svg() );
}
